# -*- coding: utf-8 -*-

# KTS 2018 Conference 발표 자료
# 2018. 02. 03
# 아무나 가져다가 마음대로 쓰세요.

import argparse
import datetime
import json
import os
import re

from openpyxl import load_workbook

pattern_cell = re.compile(r'([a-zA-Z]+)([0-9]+)')
remove_newline = re.compile(r'\n')


class KKTabu(object):
    isKKK = False  # \ifkkk가 나타나면 True로 설정하고, \fi가 나올 때까지 한줄씩 읽어들인다.
    kkk_config = ''  # isKKK=True일 때 읽어들인 텍스트를 저장한다. 이 텍스트는 각 엑셀에 대한 정보를 저장하는데, json으로 변환해서 사용한다.
    table_configs = []  # kkk_config의 리스트
    sheet = None

    def __init__(self, texfile):
        """ 메인 텍 파일에서 엑셀 정보를 가져온다.
        :param texfile: 메인 tex 파일 경로
        """
        with open(texfile) as f:
            for line in f.readlines():
                if line == '\\ifkkk\n':
                    self.isKKK = True
                elif line == '\\fi\n':
                    self.isKKK = False
                    kkk = json.loads(self.kkk_config)
                    self.table_configs.append(kkk)
                    self.kkk_config = ''
                elif self.isKKK:
                    line = remove_newline.sub('', line)
                    self.kkk_config += line

    def find_data_range(self, sheet):
        """ data 속성이 없으면 첫번째 셀과 마지막 셀을 찾는다.
        :param sheet: 시트 객체
        :type sheet: <WORKSHEET>
        :return: 표 영역
        :rtype str
        """
        x1 = chr(65 - 1 + sheet.min_column)
        x2 = chr(65 - 1 + sheet.max_column)
        y1 = sheet.min_row
        y2 = sheet.max_row
        data = '{}{}:{}{}'.format(x1, y1, x2, y2)
        return data

    def normalize_str(self, cell):
        """ 셀값을 적당히 가공한다.
        :param cell: 셀이름(A1, A2, ...)
        :type cell: str
        :return: 셀값
        :rtype str
        """
        if self.sheet[cell].value is None:
            value = ''
        elif self.sheet[cell].number_format == '0.00%' and self.sheet[cell].data_type == 'n':
            value = '{:{width}{prec}}\\%'.format(self.sheet[cell].value * 100, width=0, prec=2)
        else:
            value = str(self.sheet[cell].value)
            value = value.replace('$', '\\$')

        if self.sheet[cell].is_date:
            value = datetime.datetime.strptime(value, '%Y-%m-%d %H:%M:%S')
            value = value.strftime('%Y-%m-%d')
        return value

    def make_column_texcode(self, columns, verticalline):
        """ tabu 시작 코드에 컬럼 지정
        :param columns: 컬럼 리스트(A, B, C, ... 에 대한 아스키코드. transpose=True일 땐 행번호
        :rtype columns: list
        :param verticalline: 수직선 옵션(True)
        :rtype verticalline: bool
        :return tabu 컬럼 지시자
        :rtype str
        """
        columns_tex = '{'
        if verticalline:
            vertical_char = '|'
        else:
            vertical_char = ''

        for column in range(len(columns)):
            columns_tex += '{v}X[c]'.format(v=vertical_char)
        columns_tex += '{v}X[c]{v}}}\n'.format(v=vertical_char)
        return columns_tex

    def row_generator(self, row, columns):
        """ 열(column)을 우선으로 엑셀을 탐색해서 데이터를 가져온다.
        :param row: 행 번호
        :type row: int
        :param columns: 컬럼 리스트
        :type columns: list
        :return 열번호, 셀값
        :rtype tuple
        """
        for column in columns:
            cell = '{}{}'.format(chr(column), row)
            value = self.normalize_str(cell)
            yield (column, value)

    def column_generator(self, column, y1, y2):
        """ transpose: True 일 때,
        행을 우선으로 엑셀을 탐색해서 데이터를 가져온다.
        :param column: 알파벳의 아스키코드 번호
        :type column: int
        :param y1: 시작 행 번호
        :type y1: int
        :param y2: 마지막 행 번호
        :type y2: int
        :return 행번호, 셀값
        :rtype tuple
        """
        for row in range(y1, y2 + 1):
            cell = '{}{}'.format(chr(column), row)
            value = self.normalize_str(cell)
            yield (row, value)

    def make_export_filename(self, index, excel, export):
        """각 표에 대한 텍 파일 이름을 결정한다.
        :param index: main.tex 파일에 있는 표 정보의 순서
        :type index: int
        :param excel: 엑셀 파일명
        :type excel: str
        :param export: 테이블 텍 파일명
        :type export: str
        :return 테이블 텍 파일명
        :rtype str
        """
        file_name = '{}-{}.tex'.format(index, export)
        if export:
            if export.endswith('tex'):
                file_name = '{}-{}'.format(index, export)
        else:
            excel_name = os.path.basename(excel)
            file_name = '{}-{}.tex'.format(index, excel_name)
        return file_name

    def make_table_tex_file(self):
        """main.tex에서 가져온 테이블 정보로 엑셀 파일에서 셀값을 가져와서 텍 코드로 작성하고 파일로 저장한다.
        :return: None
        """
        for index, table_config in enumerate(self.table_configs):
            excel = table_config.get('excel')
            data = table_config.get('data')
            width = table_config.get('width')
            transpose = table_config.get('transpose') == 'True' or False
            sheet_name = table_config.get('sheet')
            verticalline = table_config.get('verticalline') == 'True' or False
            export = table_config.get('export')

            wb = load_workbook(excel, read_only=True)
            if sheet_name:
                sheet = wb.get_sheet_by_name(sheet_name)
            else:
                sheet = wb.worksheets[0]
            self.sheet = sheet

            if data is None:
                data = self.find_data_range(sheet)

            # A1, A2 등의 셀 번호를 openpyxl이 처리할 수 있는 형태로 변환한다.
            start_cell, last_cell = data.split(':')
            m1 = pattern_cell.match(start_cell)
            x1, y1 = m1.groups()
            m2 = pattern_cell.match(last_cell)
            x2, y2 = m2.groups()
            y1 = int(y1)
            y2 = int(y2)

            if transpose:
                columns = [x for x in range(y1, y2)]
            else:
                columns = [x for x in range(ord(x1), ord(x2) + 1)]
            columns_tex = self.make_column_texcode(columns, verticalline)

            if width is None:
                width = '\linewidth'
            table_tex = '\\begin{{tabu}} to {} {}'.format(width, columns_tex)

            new_row = '\\\\\n'

            if transpose:  # 행열을 바꿔서 텍 코드를 작성한다.
                for column in range(ord(x1), ord(x2) + 1):
                    for row, value in self.column_generator(column, y1, y2):
                        if row != y2:
                            table_tex += value + ' &'
                        else:
                            table_tex += value
                    if chr(column) != x2:
                        table_tex += new_row
            else:
                for row in range(y1, y2 + 1):
                    for column, value in self.row_generator(row, columns):
                        if chr(column) != x2:
                            table_tex += value + ' &'
                        else:
                            table_tex += value
                    if row != y2:
                        table_tex += new_row

            table_tex += '\n\\end{tabu}\n'
            file_name = self.make_export_filename(index, excel, export)
            with open(file_name, 'w', encoding='utf-8') as f:
                f.write(table_tex)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('texfile', type=str, help='tex file path')
    args = parser.parse_args()
    texfile = args.texfile
    kk = KKTabu(texfile)
    kk.make_table_tex_file()
